/* -*- C++ -*-
 *
 * ---------------------------------------------------------------------
 * $Id: config.h 363 2008-07-10 15:09:44Z drory $
 * ---------------------------------------------------------------------
 *
 * Copyright (C)  Niv Drory <drory@mpe.mpg.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA 
 *
 * ---------------------------------------------------------------------
 *
 */


#ifndef __LTL_CONFIG__
#define __LTL_CONFIG__


// read autoconf output
#include <ltl/acconfig.h>

// ====================================================================

// DEFAULT BEHAVIOUR : edit these to reflect your needs
//                     (can also be set manually before includig ltl
//                     headers. set reasonable default values here)

// ====================================================================

// range checking and behaviour on range errors
//

// range checking off by default
// #define LTL_RANGE_CHECKING
//
// default is to abort on range error
// #define LTL_ABORT_ON_RANGE_ERROR
// #define LTL_THROW_ON_RANGE_ERROR
//
// #undef LTL_DEBUG_EXPRESSIONS
// #undef LTL_DEBUG_MEMORY_BLOCK

//
// Thread safety
//
// Should we guard reference counting with mutex?
// This needs to be defined when running in a multithreaded
// setting, e.g. using OpenMP.
#ifdef LTL_MULTITHREAD
#  define LTL_THREADSAFE_MEMBLOCK
#endif

//
// Handling of loops in expression templates
//
// Should loops be unrolled by the LTL (see ltl/marray/eval.h)
//
#ifndef LTL_DONT_UNROLL_EXPRESSIONS
#  define LTL_UNROLL_EXPRESSIONS
#endif
// And for vectorized loops
#ifndef LTL_DONT_UNROLL_EXPRESSIONS_SIMD
#  define LTL_UNROLL_EXPRESSIONS_SIMD
#endif
//
// limit for loop unrolling in FVector and FMatrix classes
//
#ifndef LTL_TEMPLATE_LOOP_LIMIT
#  define LTL_TEMPLATE_LOOP_LIMIT 16
#endif

//
// vectorize loops using Altivec (PPC) or SSE2+ (i386)
//
//#  define LTL_USE_SIMD


// emit prefetch instructions (with GCC)
//
//#define LTL_EMIT_PREFETCH_GCC

// use namespaces 
//
#ifdef HAVE_NAMESPACES
#  define LTL_USING_NAMESPACE
#  define UTIL_USING_NAMESPACE
#endif

// ====================================================================

// END OF USER PART - the rest should not be changed

// ====================================================================



// exactly one of these two MUST be defined if we have range checking on
// either LTL_ABORT_ON_RANGE_ERROR or LTL_THROW_ON_RANGE_ERROR
//
#if defined(LTL_RANGE_CHECKING) && !defined(LTL_ABORT_ON_RANGE_ERROR) && !defined(LTL_THROW_ON_RANGE_ERROR)
#  define LTL_ABORT_ON_RANGE_ERROR
#endif


/* support for proposed 'restrict' keyword  */
#ifndef HAVE_NCEG_RESTRICT
#  define restrict_
#endif


#ifdef LTL_USING_NAMESPACE
#  define LTL_BEGIN_NAMESPACE  namespace ltl {
#  define UTIL_BEGIN_NAMESPACE namespace util {
#  define LTL_END_NAMESPACE  }
#  define UTIL_END_NAMESPACE }
#else
#  define LTL_BEGIN_NAMESPACE
#  define UTIL_BEGIN_NAMESPACE
#  define LTL_END_NAMESPACE
#  define UTIL_END_NAMESPACE
#endif


#if defined(HAVE_IEEE_MATH) && !defined(__SUNPRO_CC) && !defined(__GNUC__)
#  ifndef _ALL_SOURCE
#    define _ALL_SOURCE
#  endif
#  ifndef _XOPEN_SOURCE
#    define _XOPEN_SOURCE
#  endif
#  ifndef _XOPEN_SOURCE_EXTENDED
#    define _XOPEN_SOURCE_EXTENDED 1
#  endif
#endif


#if defined( __GNUC__ ) && defined( LTL_EMIT_PREFETCH_GCC ) 
#  if ( __GNUC__ > 3 ) || ( __GNUC__ == 3 && __GNUC_MINOR__ >= 1 )
#    define LTL_PREFETCH_R(addr) __builtin_prefetch((addr),0,1)
#    define LTL_PREFETCH_RW(addr) __builtin_prefetch((addr),1,1)
#  endif
#else
#  define LTL_PREFETCH_R(addr)
#  define LTL_PREFETCH_RW(addr)
#endif


#if defined(LTL_THREADSAFE_MEMBLOCK)

#  if  defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 1 ))
     /* GCC >= 4.2 has OpenMP and atomic memory access builtins ... */
#    define LTL_USE_GCC_ATOMIC_BUILTINS
#  elif (defined(__INTEL_COMPILER) || defined(__ICC)) && (defined(_M_IA64) || defined(__IA64__))
     /* ICC defines essentially the same builtins */
#    define LTL_USE_ICC_ATOMIC_BUILTINS
#  else
     /* default to using POSIX mutex */
#    define LTL_USE_POSIX_MUTEX
#  endif
 
#endif


#endif //__LTL_CONFIG__
